% Main Fuction %
function []=skm_2dof_cal1()
clc; 
D2R=pi/180; R2D=180/pi;
a1= 100; a2=100;
% [alpha, a, d] %
param=[0,a1,0; 0,a2,0];
 
% (1) Forward Kinematics %
th=D2R*[0,90]';
T02=forward_kinematics(th, param);
pos=T02(1:2,4);
 
disp('(1) Forward Kinematics:');
disp('Joint angles [deg]:');
disp(R2D*th');
disp('Position [mm]:');
disp(pos');
 
% (2) Inverse Kinematics %
pos=[100, 100]';
[th]=inverse_kinematics(pos, param);
disp('(2) Inverse Kinematics:');
disp('Position [mm]:');
disp(pos');
disp('Joint angles [deg]:');
disp(R2D*th');
 
% Forward & Inverse Kinematics %
%-------- Forward Kinematics
function T=forward_kinematics(th, param)
T=Transformation(th(1,1), param(1,:))*Transformation(th(2,1), param(2,:));
%-------- Inverse Kinematics
function [th, w_index]=inverse_kinematics(pos, param)
w_index=1;  % In workspace %
a1=param(1,2); a2=param(2,2);
 
% (1) Calculate th2 %
px=pos(1,1); py=pos(2,1);
kapha=(px^2+py^2-a1^2-a2^2)/(2*a1*a2);
if abs(kapha)>1
    w_index=-1;  % Out of Workspace %
    kapha=1;
end
th2=+acos(kapha); % Elbow Down %
%th2=-acos(kapha); % Elbow Up %
 
% (2) Calculate th1 %
delta=a1^2+a2^2+2*a1*a2*cos(th2);
cth1=(px*(a1+a2*cos(th2))+py*a2*sin(th2))/delta;
sth1=(-px*a2*sin(th2)+py*(a1+a2*cos(th2)))/delta;
th1=atan2(sth1, cth1); 
th=[th1, th2]';
 
% Homogeneous Transformation Matrix %
%-------- D-H Transformation Matrix
function [T]=Transformation(th, param)
alpha=param(1); a=param(2); d=param(3);
T=Trans([0,0,d]')*Rotz(th)*Trans([a,0,0]')*Rotx(alpha);
%------- Rotation matrix about the X-axis 
function [T]=Rotx(th)
cx=cos(th); sx=sin(th);
T=[1,0,0,0; 0,cx,-sx,0; 0,sx,cx,0; 0,0,0,1];
%------- Rotation matrix about the Z-axis 
function [T]=Rotz(th)
cz=cos(th); sz=sin(th);
T=[cz,-sz,0,0; sz,cz,0,0; 0,0,1,0; 0,0,0,1];
%-------- Translation along the x, y, z axes 
function [T]=Trans(p)
T=[1,0,0,p(1,1); 0,1,0,p(2,1); 0,0,1,p(3,1); 0,0,0,1];
